import * as React from 'react';
import { Feature } from '@common-stack/client-react';
import { Files, /** Editors */ } from '@files-stack/client-react';
import { reducers, setRootPath, filesModule } from '@files-stack/client-state';
import {
    filesResolvers, defaults, editorResolvers, filesTreeResolvers, filesSchema, dataIdFromObject,
    modelContentChangeEpic, modelSaveEpic, editorSchema, filesTreeSchema, fragmentMatcher,
    untitledEditorSchema,
} from '@files-stack/client-state';
import { merge } from 'lodash';
import { IClientFileService, ClientTypes, IFilesTreeService, IUntitledEditorService,
     IClientService, IClientTextFileService,
     } from '@files-stack/core';
import { interfaces } from 'inversify';

const actionFunctions = [setRootPath];

const filesServiceGen = (container: interfaces.Container): IClientService => {
    return {
        fileService: container.get<IClientFileService>(ClientTypes.IFileService),
        textFileService: container.get<IClientTextFileService>(ClientTypes.ITextFileService),
        filesTreeService: container.get<IFilesTreeService>(ClientTypes.IFilesTreeService),
        untitledEditorService: container.get<IUntitledEditorService>(ClientTypes.IUntitledEditorService),
    };
};

export default new Feature({
    leftMainPanelItems: { files: <Files /> },
    middleMainPanelItems: { editors: <div>ToDO</div>  }, // <Editor/>
    middleMainPanelItemsProps: { editors: { actionFunctions } },
    reducer: { ...reducers },
    createContainerFunc: filesModule,
    clientStateParams: [{ resolvers: merge(filesResolvers, editorResolvers, filesTreeResolvers), defaults, 
        typeDefs: [filesSchema, untitledEditorSchema, editorSchema, filesTreeSchema], fragmentMatcher: fragmentMatcher as any }],
    dataIdFromObject,
    createServiceFunc: filesServiceGen,
    epic: [modelContentChangeEpic, modelSaveEpic],
});
